package com.bookcollector.model.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import com.bookcollector.model.Author;
import com.bookcollector.model.Book;
import com.bookcollector.model.Category;
import com.bookcollector.model.Publisher;
import com.bookcollector.tool.ConnectionManager;

public class BookDAO {
	Connection conn = ConnectionManager.getConnection();

	public ArrayList<Book> getAllBooks(){
		ArrayList<Book> books = new ArrayList<>();
		
		String query = "SELECT * FROM Book";
		
		try {
			PreparedStatement psmt = conn.prepareStatement(query);
			ResultSet rs = psmt.executeQuery();
			while(rs.next()){
				Author author = new AuthorDAO().getAuthor(rs.getInt("Author_idAuthor"));
				Category category = new CategoryDAO().getCategory(rs.getInt("Category_idCategory"));
				Publisher publisher = new PublisherDAO().getPublisher(rs.getInt("Publisher_idPublisher"));
				String name = rs.getString("name");
				int totalPages = rs.getInt("totalPages");
				long ISBN = rs.getLong("ISBN");
				int idBook = rs.getInt("idBook");
				books.add(new Book(idBook, category, author, publisher, ISBN, totalPages, name));
			}
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
		return books;
	}
	
	public boolean addBook(Book book){
		String query = "INSERT INTO Book VALUES (DEFAULT, ?, ?, ?, ?, ?, ?)";
		
		try{
			PreparedStatement psmt = conn.prepareStatement(query);
			psmt.setString(1, book.getName());
			psmt.setInt(2, book.getTotalPages());
			psmt.setLong(3, book.getISBN());
			psmt.setInt(4, book.getPublisher().getIdPublisher());
			psmt.setInt(5, book.getAuthor().getIdAuthor());
			psmt.setInt(6, book.getCategory().getIdCategory());
			psmt.executeUpdate();
		}catch(SQLException ex){
			ex.printStackTrace();
		}
		
		return true;
	}
	
	public boolean deleteBook(int id){
		String query = "DELETE FROM Book WHERE idBook=?";
		
		try{
			PreparedStatement psmt = conn.prepareStatement(query);
			psmt.setInt(1, id);
			psmt.executeUpdate();
		}catch(SQLException ex){
			ex.printStackTrace();
		}
		
		return true;
	}
	
	public Book getBook(int idBook){
		
		String query = "SELECT * FROM Book WHERE idBook=?";
		Book book = null;
		
		try {
			PreparedStatement psmt = conn.prepareStatement(query);
			psmt.setInt(1, idBook);
			ResultSet rs = psmt.executeQuery();
			while(rs.next()){
				Author author = new AuthorDAO().getAuthor(rs.getInt("Author_idAuthor"));
				Category category = new CategoryDAO().getCategory(rs.getInt("Category_idCategory"));
				Publisher publisher = new PublisherDAO().getPublisher(rs.getInt("Publisher_idPublisher"));
				String name = rs.getString("name");
				int totalPages = rs.getInt("totalPages");
				long ISBN = rs.getLong("ISBN");
				book = new Book(idBook, category, author, publisher, ISBN, totalPages, name);
			}
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
		return book;
	}
	
	public boolean updateBook(Book book){
		String query = "UPDATE Book SET name=?, totalPages=?, ISBN=?, " +
				"Publisher_idPublisher=?, Author_idAuthor=?, " +
				"Category_idCategory=? WHERE idBook=?";
		
		try{
			PreparedStatement psmt = conn.prepareStatement(query);
			psmt.setString(1, book.getName());
			psmt.setInt(2, book.getTotalPages());
			psmt.setLong(3, book.getISBN());
			psmt.setInt(4, book.getPublisher().getIdPublisher());
			psmt.setInt(5, book.getAuthor().getIdAuthor());
			psmt.setInt(6, book.getCategory().getIdCategory());
			psmt.setInt(7, book.getIdBook());
			psmt.executeUpdate();
		}catch(SQLException ex){
			ex.printStackTrace();
		}
		
		return true;
	}
	
	public ArrayList<Book> getLastestBooks(){
		ArrayList<Book> books = new ArrayList<>();
		
		String query = "SELECT * FROM Books.Book ORDER BY idBook DESC LIMIT 5;";
		
		try {
			PreparedStatement psmt = conn.prepareStatement(query);
			ResultSet rs = psmt.executeQuery();
			while(rs.next()){
				Author author = new AuthorDAO().getAuthor(rs.getInt("Author_idAuthor"));
				Category category = new CategoryDAO().getCategory(rs.getInt("Category_idCategory"));
				Publisher publisher = new PublisherDAO().getPublisher(rs.getInt("Publisher_idPublisher"));
				String name = rs.getString("name");
				int totalPages = rs.getInt("totalPages");
				long ISBN = rs.getLong("ISBN");
				int idBook = rs.getInt("idBook");
				books.add(new Book(idBook, category, author, publisher, ISBN, totalPages, name));
			}
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
		return books;
	}
}
